/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.FrameBufferPanel;

import java.awt.Color;
import javax.swing.JFrame;
import java.awt.BorderLayout;
import java.io.File;
import java.awt.event.KeyEvent;

/**

*/
public class InteractiveModelsAll_R6 extends InteractiveAbstractClient_R6
{
   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
      Then this constructor instantiates the GUI.
   */
   public InteractiveModelsAll_R6()
   {
      // Create the Scene object that we shall render.
      scene = new Scene();
/*
      // Create several Model objects, using default constructors.
      modelArray.add(new Square());
      modelArray.add(new SquareGrid());
      modelArray.add(new Circle());
      modelArray.add(new Disk());
      modelArray.add(new DiskSector());
      modelArray.add(new Ring());
      modelArray.add(new RingSector());
      modelArray.add(new Box());
      modelArray.add(new Cube());
      modelArray.add(new Cube2());
      modelArray.add(new Cube3());
      modelArray.add(new Cube4());
      modelArray.add(new Tetrahedron());
      modelArray.add(new Octahedron());
      modelArray.add(new Icosahedron());
      modelArray.add(new Dodecahedron());
      modelArray.add(new Pyramid());
      modelArray.add(new PyramidFrustum());
      modelArray.add(new TriangularPyramid());
      modelArray.add(new TriangularPrism());
      modelArray.add(new ViewFrustum());
      modelArray.add(new Cone());
      modelArray.add(new ConeSector());
      modelArray.add(new ConeFrustum());
      modelArray.add(new Cylinder());
      modelArray.add(new CylinderSector());
      modelArray.add(new Sphere());
      modelArray.add(new SphereSector());
      modelArray.add(new SphereSubdivided());
      modelArray.add(new Torus());
      modelArray.add(new TorusSector());
      modelArray.add(new ParametricCurve());
      modelArray.add(new ParametricSurface());
      modelArray.add(new SurfaceOfRevolution());
      modelArray.add(new Axes3D());
      modelArray.add(new Axes2D());
      modelArray.add(new PanelXY());
*/
      // Instantiate at least one of every Model class.
      // 2D models
      modelArray.add(new Square(1.0));
      modelArray.add(new SquareGrid(1.0, 11, 15));
      modelArray.add(new Circle(1.0, 16));
      modelArray.add(new Disk(1.0, 4, 16));
      modelArray.add(new DiskSector(1.0, Math.PI/2, 3*Math.PI/2, 4, 8));
      modelArray.add(new Ring(1.0, 0.25, 3, 16));
      modelArray.add(new RingSector(1.0, 0.25, Math.PI/2, 3*Math.PI/2, 3, 8));
      modelArray.add(new Triangle(45, 4)); // try 5 or 6
      // cubes
      modelArray.add(new Box(1.0, 1.0, 1.0));
      modelArray.add(new Cube( ));
      modelArray.add(new Cube2(4, 5, 6));
      modelArray.add(new Cube3(12, 14, 15));
      modelArray.add(new Cube4(12, 14, 15));
      // polyhedra
      modelArray.add(new Tetrahedron());
      modelArray.add(new Tetrahedron(true));
      modelArray.add(new Octahedron());
      modelArray.add(new Icosahedron());
      modelArray.add(new Dodecahedron());
      // pyramids
      modelArray.add(new Pyramid(2.0, 1.0, 5, 6));
      modelArray.add(new Pyramid(2.0, 1.0, 5, 6, true));
      modelArray.add(new PyramidFrustum(2.0, 1.0, 0.5, 2, 5));
      modelArray.add(new PyramidFrustum(1.0, 2.0, 0.5, 2, 5));
      modelArray.add(new TriangularPyramid(Math.sqrt(3)/Math.sqrt(2)));
      modelArray.add(new TriangularPyramid(1.0, 1.0, 7, 7));
      modelArray.add(new TriangularPrism(0.6, 0.5, 0.5, 3, true));
      modelArray.add(new ViewFrustumModel());
      // cones
      modelArray.add(new Cone(1.0, 1.0, 10, 16));
      modelArray.add(new ConeSector(1.0, 1.0, 0.5, 0, 2*Math.PI, 5, 16));
      modelArray.add(new ConeSector(1.0, 1.0, 0.5, Math.PI/2, 3*Math.PI/2, 5, 8));
      modelArray.add(new ConeFrustum(1.0, 0.5, 0.5, 6, 16));
      modelArray.add(new ConeFrustum(0.5, 1.0, 0.5, 6, 16));
      // cylinders
      modelArray.add(new Cylinder(0.5, 1.0, 11, 12));
      modelArray.add(new CylinderSector(0.5, 1.0, Math.PI/2, 3*Math.PI/2, 11, 6));
      // spheres
      modelArray.add(new Sphere(1.0, 15, 12));
      modelArray.add(new SphereSector(1.0, Math.PI/2, 3*Math.PI/2,
                                           Math.PI/4, 3*Math.PI/4, 7, 6));
      modelArray.add(new SphereSubdivided(4));
      modelArray.add(new SphereSubdivided(6, true, false));
      modelArray.add(new SphereSubdivided(7, false, true));
      // torus
      modelArray.add(new Torus(0.75, 0.25, 12, 16));
      modelArray.add(new TorusSector(0.75, 0.25, Math.PI/2, 3*Math.PI/2, 12, 8));
      modelArray.add(new TorusSector(0.75, 0.25, 0, 2*Math.PI,
                                                      Math.PI, 2*Math.PI, 6, 16));
      modelArray.add(new TorusSector(0.75, 0.25, 0, 2*Math.PI,
                                                     -Math.PI/2, Math.PI/2, 6, 16));
      modelArray.add(new TorusSector(0.75, 0.25, Math.PI/2, 3*Math.PI/2,
                                                -Math.PI/2, Math.PI/2, 6, 8));
      // model files
      modelArray.add(new GRSModel(new File("assets/grs/bronto.grs")));
      modelArray.add(new ObjSimpleModel(new File("assets/cow.obj")));
      // parametric curves and surfaces
      modelArray.add(new ParametricCurve());
      modelArray.add(new ParametricSurface());
      modelArray.add(new ParametricSurface(
                (s,t)->s*Math.cos(t*Math.PI),
                (s,t)->t,
                (s,t)->s*Math.sin(t*Math.PI),
                -1, 1, -1, 1, 49, 49));
      modelArray.add(new ParametricSurface(
                (u,v)->0.3*(1-u)*(3+Math.cos(v))*Math.sin(4*Math.PI*u),
                (u,v)->0.3*(3*u+(1-u)*Math.sin(v)),
                (u,v)->0.3*(1-u)*(3+Math.cos(v))*Math.cos(4*Math.PI*u),
                0, 1, 0, 2*Math.PI, 49, 49));
      modelArray.add(new SurfaceOfRevolution());
      modelArray.add(new SurfaceOfRevolution(t->0.5*(1+t*t), -1, 1, 30, 30));
      modelArray.add(new SurfaceOfRevolution(t->t, t->4*t*(1-t), 0, 1, 30, 30));
      // coordinate axes
      modelArray.add(new Axes3D(1, 1, 1));
      modelArray.add(new Axes2D(-1, 1, -1, 1, 8, 8));
      modelArray.add(new PanelXY(-4, 4, -5, 5, -5));
      modelArray.add(new PanelXZ(-1, 1, -6, 1, -0.5));
      modelArray.add(new PanelYZ(-1, 1, -6, 1, -0.5));

      // Give each model a random color.
      for (final Model m : modelArray)
      {
         ModelShading.setRandomColor(m);
      }

      // Add a model to the Scene.
      scene.addModel(modelArray.get(currentModel));

      // Push the models away from where the camera is.
      for (final Model m : modelArray)
      {
         for (int i = 0; i < m.vertexList.size(); ++i)
         {
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x,
                                           v.y,
                                           v.z - 2));
         }
      }


      // Define initial dimensions for a FrameBuffer.
      final int width  = 1024;
      final int height = 1024;

      // Create a FrameBufferPanel that will hold a FrameBuffer.
      fbp = new FrameBufferPanel(width, height);
      fbp.getFrameBuffer().setBackgroundColorFB(Color.darkGray);
      fbp.getFrameBuffer().getViewport().setBackgroundColorVP(Color.black);

      // Create a JFrame that will hold the FrameBufferPanel.
      jf = new JFrame("Renderer 6");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    //jf.setLocationByPlatform(true);
      // Place the FrameBufferPanel in the JFrame.
      jf.getContentPane().add(fbp, BorderLayout.CENTER);
      jf.pack();
      jf.setVisible(true);

      // Create event handler objects for events from the JFrame.
      jf.addKeyListener(this);
      jf.addComponentListener(this);
   }


   // Change how the program deals with transformations.
   @Override protected void doTransformations(char c)
   {
      if ('s' == c) // Scale ALL the models 10% smaller.
      {
         deltaS[0] /= 1.1;
         scaleModels(1.0/1.1);
      }
      else if ('S' == c) // Scale ALL the models 10% larger.
      {
         deltaS[0] *= 1.1;
         scaleModels(1.1);
      }
      else if ('x' == c) // Translate ALL the models.
      {
         deltaX[0] -= 0.1;
         moveModels(-0.1, 0.0, 0.0);
      }
      else if ('X' == c)
      {
         deltaX[0] += 0.1;
         moveModels(+0.1, 0.0, 0.0);
      }
      else if ('y' == c)
      {
         moveModels(0.0, -0.1, 0.0);
      }
      else if ('Y' == c)
      {
         deltaY[0] += 0.1;
         moveModels(0.0, +0.1, 0.0);
      }
      else if ('z' == c)
      {
         deltaZ[0] -= 0.1;
         moveModels(0.0, 0.0, -0.1);
      }
      else if ('Z' == c)
      {
         deltaZ[0] += 0.1;
         moveModels(0.0, 0.0, +0.1);
      }
   }

   protected void scaleModels(double deltaS)
   {
      for (Model m : modelArray)
      {
         for (int i = 0; i < m.vertexList.size(); ++i)
         {
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x * deltaS,
                                           v.y * deltaS,
                                           v.z * deltaS));
         }
      }
   }


   protected void moveModels(double deltaX, double deltaY, double deltaZ)
   {
      for (Model m : modelArray)
      {
         for (int i = 0; i < m.vertexList.size(); ++i)
         {
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x + deltaX,
                                           v.y + deltaY,
                                           v.z + deltaZ));
         }
      }
   }


   // Change how the program prints transformations.
   @Override protected void displayTransformations(char c)
   {
      if ('m'==c
        ||'s'==c||'x'==c||'y'==c||'z'==c
        ||'S'==c||'X'==c||'Y'==c||'Z'==c)
      {
         System.out.printf("deltaX = %.2f, deltaY = %.2f, " +
                           "deltaZ = %.2f, deltaS = %.2f\n",
                            deltaX[0],     deltaY[0],
                            deltaZ[0],     deltaS[0]);
      }
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> {new InteractiveModelsAll_R6();}
      );
   }
}
