/*

*/

import renderer.scene.*;
import renderer.models.Disk;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import javax.swing.JFrame;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ComponentEvent;
import java.awt.BorderLayout;

/**

*/
public class TestAspectRatios implements KeyListener, ComponentListener
{
   protected final int SIZE = 512;

   private final Scene scene;
   private final JFrame jf;
   private final FrameBufferPanel fbp;

   public TestAspectRatios()
   {
      scene = new Scene();

      final double left   = -1.0;
      final double right  =  1.0;
      final double bottom = -1.0;
      final double top    =  1.0;
      scene.camera.projOrtho(left, right, bottom, top);

      final Model model = new Disk(1.0, 10, 40);

      ModelShading.setRandomColor(model);

      scene.addModel(model);

      // Define initial dimensions for a FrameBuffer.
      final int width  = SIZE;
      final int height = SIZE;

      // Create a FrameBufferPanel that will hold a FrameBuffer.
      fbp = new FrameBufferPanel(width, height);

      // Create a JFrame that will hold the FrameBufferPanel.
      jf = new JFrame("Renderer 6");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    //jf.setLocationByPlatform(true);
      // Place the FrameBufferPanel in the JFrame.
      jf.getContentPane().add(fbp, BorderLayout.CENTER);
      jf.pack();
      jf.setVisible(true);

      // Create event handler objects for events from the JFrame.
      jf.addKeyListener(this);
      jf.addComponentListener(this);
   }


   // Implement the KeyListener interface.
   @Override public void keyPressed(KeyEvent e){}
   @Override public void keyReleased(KeyEvent e){}
   @Override public void keyTyped(KeyEvent e)
   {
   }

   // Implement the ComponentListener interface.
   @Override public void componentMoved(ComponentEvent e){}
   @Override public void componentHidden(ComponentEvent e){}
   @Override public void componentShown(ComponentEvent e){}
   @Override public void componentResized(ComponentEvent e)
   {
      //System.out.println( e );

      // Get the new size of the JFrame.
      final int wJF = jf.getWidth();
      final int hJF = jf.getHeight();

      // Get the new size of the JFrame's content pane.
      final int wCP = jf.getContentPane().getWidth();
      final int hCP = jf.getContentPane().getHeight();

      // Get the new size of the FrameBufferPanel.
      final int wFBP = fbp.getWidth();
      final int hFBP = fbp.getHeight();

      // Get the (old) size of the FrameBuffer.
      final FrameBuffer fb = fbp.getFrameBuffer();
      final int wFB = fb.width;
      final int hFB = fb.height;

      // Create a new FrameBuffer that fits the FrameBufferPanel.
//      fb = new FrameBuffer(wFBP, hFBP);
//      fbp.setFrameBuffer(fb);

      // The view volume is centered on left edge.
      scene.camera.projOrtho(-1,                        // left
                             -1 + (2.0 * wFBP) / SIZE,  // right
                             -hFBP / (double)SIZE,      // bottom
                              hFBP / (double)SIZE);     // top

      System.out.printf("JFrame [w = %d, h = %d] w/h = %.2f.\n",
                        wJF, hJF, (double)wJF / (double)hJF);

      System.out.printf("ContentPane [w = %d, h = %d] w/h = %.2f.\n",
                        wCP, hCP, (double)wCP / (double)hCP);

      System.out.printf("JPanel [w = %d, h = %d] w/h = %.2f.\n",
                        wFBP, hFBP, (double)wFBP / (double)hFBP);

      System.out.printf("Framebuffer [w = %d, h = %d] w/h = %.2f.\n",
                        wFB, hFB, (double)wFB / (double)hFB);

      System.out.print( scene.camera );
      System.out.println("=================================");

      // Render again.
      fb.clearFB();
      fb.vp.clearVP();
      Pipeline.render(scene, fb);
      fbp.update();
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new TestAspectRatios()
      );
   }
}
