/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;
import javax.swing.JFrame;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ComponentEvent;

/**

*/
public abstract class Circle_v0_Abstract
       implements KeyListener, ComponentListener
{
   protected final int SIZE = 512;

   protected boolean showCamera = false;
   protected boolean showFBaspectRatio = false;
   protected int mode = 1;

   protected Scene scene = null;
   protected FrameBufferPanel fbp = null;


   // Implement the KeyListener interface.
   @Override public void keyPressed(KeyEvent e){}
   @Override public void keyReleased(KeyEvent e){}
   @Override public void keyTyped(KeyEvent e)
   {
      //System.out.println( e );

      final char c = e.getKeyChar();
      if ('h' == c)
      {
         print_help_message();
         return;
      }
      else if ('d' == c)
      {
         scene.getModel(0).debug = ! scene.getModel(0).debug;
         Clip.debug = scene.getModel(0).debug;
      }
      else if ('D' == c)
      {
         RasterizeAntialias.debug = ! RasterizeAntialias.debug;
      }
      else if ('a' == c)
      {
         RasterizeAntialias.doAntialiasing = ! RasterizeAntialias.doAntialiasing;
         System.out.print("Anti-aliasing is turned ");
         System.out.println(RasterizeAntialias.doAntialiasing ? "On" : "Off");
      }
      else if ('g' == c)
      {
         RasterizeAntialias.doGamma = ! RasterizeAntialias.doGamma;
         System.out.print("Gamma correction is turned ");
         System.out.println(RasterizeAntialias.doGamma ? "On" : "Off");
      }
      else if ('m'==c || 'M' == c)
      {
         showCamera = ! showCamera;
      }
      else if ('f' == c)
      {
         showFBaspectRatio = ! showFBaspectRatio;
         if (showFBaspectRatio)
         {
            // Get the new size of the FrameBufferPanel.
            final int w = fbp.getWidth();
            final int h = fbp.getHeight();
            System.out.printf("Aspect ratio (of framebuffer) = %.2f\n", (double)w/(double)h);
         }
      }
      else if ('c' == c)
      {
         // Change the solid random color of the disk.
         ModelShading.setRandomColor(scene.getModel(0));
      }
      else if ('C' == c)
      {
         // Change each color in the disk to a random color.
         ModelShading.setRandomColors(scene.getModel(0));
      }
      else if ('e' == c && e.isAltDown())
      {
         // Change the random color of each vertex of the disk.
         ModelShading.setRandomVertexColors(scene.getModel(0));
      }
      else if ('e' == c)
      {
         // Change the solid random color of each edge of the disk.
         ModelShading.setRandomLineSegmentColors(scene.getModel(0));
      }
      else if ('E' == c)
      {
         // Change the random color of each end of each edge of the disk.
         ModelShading.setRainbowLineSegmentColors(scene.getModel(0));
      }
      else if ('1' == c)
      {
         mode = 1;
      }
      else if ('2' == c)
      {
         mode = 2;
      }
      else if ('3' == c)
      {
         mode = 3;
      }
      else if ('4' == c)
      {
         mode = 4;
      }
      else if ('5' == c)
      {
         mode = 5;
      }
      else if ('6' == c)
      {
         mode = 6;
      }
      else if ('7' == c)
      {
         mode = 7;
      }
      else if ('8' == c)
      {
         mode = 8;
      }
      else if ('9' == c)
      {
         mode = 9;
      }

      setupViewing();
   }


   // Implement the ComponentListener interface.
   @Override public void componentMoved(ComponentEvent e){}
   @Override public void componentHidden(ComponentEvent e){}
   @Override public void componentShown(ComponentEvent e){}
   @Override public void componentResized(ComponentEvent e){}


   // Get in one place the code to set up the viewport and the view volume.
   protected void setupViewing()
   {
      if (showCamera)
      {
         System.out.print( scene.camera );
      }

      // Render again.
      final FrameBuffer fb = fbp.getFrameBuffer();
      fb.clearFB();
      fb.vp.clearVP();
      Pipeline.render(scene, fb);
      fbp.update();
   }


   protected static void print_help_message()
   {
      System.out.println("Use the 'd/D' key to toggle debugging information on and off.");
      System.out.println("Use the 'a' key to toggle antialiasing on and off.");
      System.out.println("Use the 'g' key to toggle gamma correction on and off.");
      System.out.println("Use the 'f' key to toggle showing framebufer aspect ratio.");
      System.out.println("Use the 'm' key to toggle showing the Camera data.");
      System.out.println("Use the '1' through '9' keys to choose a letterbox/crop/window mode.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}
